<?php

namespace CRM\Factory;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class EnderecoFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    /**
     * Obtém os países.
     * @param array $params Parâmetros da consulta.
     * @return array Resultado da consulta.
     * @throws \Throwable Em caso de erro.
     */

    public function getCountry($params)
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder
                ->select([
                    "CDPAIS AS CDPAIS",
                    "UPPER(NMPAIS) AS NMPAIS",
                    "DTULTATUPAIS",
                    "CDDDIPAIS",
                    "CDPAISEXPFIS",
                    "CDPAISBACEN",
                    "UPPER(SGPAIS) AS SGPAIS",
                    "CDPAISGMB",
                    "NRORG",
                    "DTINCLUSAO",
                    "DTULTATU",
                    "NRORGINCLUSAO",
                    "CDOPERINCLUSAO",
                    "NRORGULTATU",
                    "CDOPERULTATU",
                    "IDATIVO"
                ])
                ->from('PAIS')
                ->where($queryBuilder->expr()->eq('NRORG', ':NRORG'))
                ->orderBy('NMPAIS', 'ASC');

            $queryBuilder->setParameter('NRORG', $params["NRORG"]);
            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function getState($params)
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder
                ->select([
                    "ESTADO.CDPAIS",
                    "PAIS.NMPAIS",
                    "ESTADO.SGESTADO",
                    "ESTADO.NMESTADO",
                    "ESTADO.CDESTAEXPO",
                    "ESTADO.DTULTATUESTA",
                    "ESTADO.CDESTADOIBGE",
                    "ESTADO.IDUTHORVER",
                    "ESTADO.HRDIFUTCFUSO",
                    "ESTADO.NRORG",
                    "ESTADO.DTINCLUSAO",
                    "ESTADO.DTULTATU",
                    "ESTADO.NRORGINCLUSAO",
                    "ESTADO.CDOPERINCLUSAO",
                    "ESTADO.NRORGULTATU",
                    "ESTADO.CDOPERULTATU",
                    "ESTADO.IDATIVO",
                    "ESTADO.CDESTADOGMB"
                ])
                ->from('ESTADO', 'ESTADO')
                ->leftJoin('ESTADO', 'PAIS', 'PAIS', 'ESTADO.CDPAIS = PAIS.CDPAIS AND ESTADO.NRORG = PAIS.NRORG')
                ->where($queryBuilder->expr()->eq('ESTADO.NRORG', ':NRORG'))
                ->orderBy('NMESTADO', 'ASC');

            $queryBuilder->setParameter('NRORG', $params["NRORG"]);
            if ($params["CDPAIS"] !== null) {
                $queryBuilder->andWhere($queryBuilder->expr()->in('PAIS.CDPAIS', ':CDPAIS'))
                    ->setParameter('CDPAIS', $params["CDPAIS"]);
            }
            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function getMunicipality($params)
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder
                ->select([
                    "MUNICIPIO.CDPAIS",
                    "PAIS.NMPAIS",
                    "MUNICIPIO.SGESTADO",
                    "ESTADO.NMESTADO",
                    "MUNICIPIO.CDMUNICIPIO",
                    "MUNICIPIO.NMMUNICIPIO",
                    "MUNICIPIO.CDMUNICIBGE",
                    "MUNICIPIO.IDMUNIALC",
                    "MUNICIPIO.CDMUNIEXPO",
                    "MUNICIPIO.DTULTATUMUNI",
                    "MUNICIPIO.CDREGIAO",
                    "MUNICIPIO.CDDDDMUNICIP",
                    "MUNICIPIO.IDMUNIUTCONVNFE",
                    "MUNICIPIO.CDMUNICIPIOGMB",
                    "MUNICIPIO.NRORG",
                    "MUNICIPIO.DTINCLUSAO",
                    "MUNICIPIO.DTULTATU",
                    "MUNICIPIO.NRORGINCLUSAO",
                    "MUNICIPIO.CDOPERINCLUSAO",
                    "MUNICIPIO.NRORGULTATU",
                    "MUNICIPIO.CDOPERULTATU",
                    "MUNICIPIO.IDATIVO",
                    "MUNICIPIO.CDMUNICSIAFI"
                ])
                ->from('MUNICIPIO', 'MUNICIPIO')
                ->leftJoin('MUNICIPIO', 'PAIS', 'PAIS', 'MUNICIPIO.CDPAIS = PAIS.CDPAIS AND MUNICIPIO.NRORG = PAIS.NRORG')
                ->leftJoin('MUNICIPIO', 'ESTADO', 'ESTADO', 'MUNICIPIO.SGESTADO = ESTADO.SGESTADO AND MUNICIPIO.NRORG = ESTADO.NRORG')
                ->where(
                    $queryBuilder->expr()->and(
                        $queryBuilder->expr()->in('MUNICIPIO.NRORG', ':NRORG'),
                    )
                )
                ->orderBy('MUNICIPIO.NMMUNICIPIO', 'ASC');

            $queryBuilder->setParameter('NRORG', $params["NRORG"]);
            if ($params["CDPAIS"] !== null && $params["SGESTADO"] !== null) {
                $queryBuilder->andWhere($queryBuilder->expr()->in('PAIS.CDPAIS', ':CDPAIS'))
                    ->andWhere($queryBuilder->expr()->in('ESTADO.SGESTADO', ':SGESTADO'))
                    ->setParameter('CDPAIS', $params["CDPAIS"], Connection::PARAM_STR_ARRAY)
                    ->setParameter('SGESTADO', $params["SGESTADO"], Connection::PARAM_STR_ARRAY);
            }
            $result = $this->createAndExecuteQueryAndBuildResponseData(
                $this->entityManager,
                $queryBuilder->getSQL(),
                $queryBuilder->getParameters()
            );

            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function getNeighborhood($params)
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder
                ->select([
                    "BAIRRO.NRORG",
                    "BAIRRO.CDPAIS",
                    "BAIRRO.SGESTADO",
                    "BAIRRO.CDMUNICIPIO",
                    "BAIRRO.CDREGIAOMUNI",
                    "BAIRRO.CDBAIRRO",
                    "BAIRRO.NMBAIRRO",
                    "BAIRRO.VRTXENTREGA",
                    "BAIRRO.QTHORAENTREGA",
                    "REGIAOMUNI.NMREGIAOMUNI"
                ])
                ->from('BAIRRO', 'BAIRRO')
                ->leftJoin('BAIRRO', 'REGIAOMUNI', 'REGIAOMUNI', 'BAIRRO.CDPAIS = REGIAOMUNI.CDPAIS AND BAIRRO.SGESTADO = REGIAOMUNI.SGESTADO AND BAIRRO.CDMUNICIPIO = REGIAOMUNI.CDMUNICIPIO AND BAIRRO.CDREGIAOMUNI = REGIAOMUNI.CDREGIAOMUNI AND BAIRRO.NRORG = REGIAOMUNI.NRORG')
                ->where(
                    $queryBuilder->expr()->and(
                        $queryBuilder->expr()->eq('BAIRRO.NRORG', ':NRORG'),
                    )
                )
                ->orderBy('BAIRRO.NMBAIRRO', 'ASC');

            $queryBuilder->setParameter('NRORG', $params["NRORG"]);

            $queryBuilder->setParameter('NRORG', $params["NRORG"]);
            if ($params["CDPAIS"] !== null && $params["SGESTADO"] !== null && $params["CDMUNICIPIO"] !== null) {
                $queryBuilder->andWhere($queryBuilder->expr()->in('BAIRRO.CDPAIS', ':CDPAIS'))
                    ->andWhere($queryBuilder->expr()->in('BAIRRO.SGESTADO', ':SGESTADO'))
                    ->andWhere($queryBuilder->expr()->in('BAIRRO.CDMUNICIPIO', ':CDMUNICIPIO'))
                    ->setParameter('CDPAIS', $params["CDPAIS"], Connection::PARAM_STR_ARRAY)
                    ->setParameter('SGESTADO', $params["SGESTADO"], Connection::PARAM_STR_ARRAY)
                    ->setParameter('CDMUNICIPIO', $params["CDMUNICIPIO"], Connection::PARAM_STR_ARRAY);
            }

            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function findCodeDistrict($parametros)
    {

        $queryBuilder = $this->connection->createQueryBuilder();
        $queryBuilder
            ->select([
                "CDBAIRRO",
                "NMBAIRRO",
                "SGESTADO"
            ])
            ->from('BAIRRO')
            ->where(
                $queryBuilder->expr()->and(
                    $queryBuilder->expr()->like('NMBAIRRO', ':NMBAIRRO'),
                    $queryBuilder->expr()->eq('SGESTADO', ':SGESTADO'),
                    $queryBuilder->expr()->eq('CDPAIS', ':CDPAIS'),
                    $queryBuilder->expr()->eq('CDMUNICIPIO', ':CDMUNICIPIO'),
                    $queryBuilder->expr()->eq('IDATIVO', ':IDATIVO')
                )
            );

        $queryBuilder->setParameter('NMBAIRRO', $parametros["NMBAIRRO"]);
        $queryBuilder->setParameter('SGESTADO', $parametros["SGESTADO"]);
        $queryBuilder->setParameter('CDPAIS', $parametros["CDPAIS"]);
        $queryBuilder->setParameter('CDMUNICIPIO', $parametros["CDMUNICIPIO"]);
        $queryBuilder->setParameter('IDATIVO', 'S');

        $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
        return $result;
    }
}
