import {
  Http,
  IDictionary,
  IEventParam,
  Metadata,
} from '@zeedhi/core';
import {
  Button,
  DialogService,
  Form,
  LoadingService,
  Modal,
  ModalService,
  Select,
  SelectMultiple,
  Tabs,
  TextInput,
} from '@zeedhi/common';
import { TekGrid, TekRestDatasource, TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

/**
 * Represents a class for managing CRM leads.
 */
export class CrmLeads {
  /**
   * Represents a modal element used for adding new leads.
   *
   * This property holds a reference to a modal element that is used for adding new leads.
   * It is of type Modal, which represents a modal window or dialog.
   *
   * @type {Modal} A modal element for adding new leads.
   */
  private ModalElementoAdd!: Modal;

  public currentRow: any = {};

  public tekgridVisibleLeads: boolean = true;

  public tekgridVisibleEvolution: boolean = true;

  public buttonQualifiedyVisibility: boolean = true;

  public buttonDesqualifiedyVisibility: boolean = true;

  get textFrameLeadName() {
    return this.currentRow.NMCANDIDATO;
  }

  /**
   * Retrieves the code of the currently logged operator.
   *
   * This method asynchronously fetches the code of the operator who is currently logged in.
   * It returns a Promise that resolves to a string representing the code of the logged operator.
   *
   * @returns {Promise<string>} A Promise that resolves to the code of the logged operator.
   */
  private async getLoggedOperator(): Promise<string> {
    let loggedOper: string = '';
    try {
      const response = await Utilities.getLoggedOperator();
      loggedOper = response.data.CDOPERADOR;
    } catch (error) {
      Utilities.showErrorMessage(error);
    }
    return loggedOper;
  }

  /**
   * Closes the modal for adding new leads.
   */
  public closeModalCadastroLeads() {
    this.ModalElementoAdd.destroy();
  }

  get formVisibleLeads() {
    return !this.tekgridVisibleLeads;
  }

  public backTekGridLeads() {
    this.tekgridVisibleLeads = true;
  }

  public backTekGridEvolution() {
    this.tekgridVisibleEvolution = true;
  }

  public openLeadsFrame({ row }: any) {
    this.hideLeadGrid();
    this.selectLeadTab();
    this.currentRow = row;
    this.loadTabLeadsData(row);
    this.modifiVisibilityBasedOnStatusLead(row);
  }

  public openEvolutionInternalFrame({ row }: any) {
    this.currentRow = row;
    this.hideEvolutionGrid();
  }

  private selectLeadTab() {
    const tabs = Metadata.getInstance<Tabs>('tabsInformationLeads');
    tabs.activeTab = 0;
  }

  private hideLeadGrid() {
    this.tekgridVisibleLeads = false;
  }

  private hideEvolutionGrid() {
    this.tekgridVisibleEvolution = false;
  }

  private loadTabLeadsData(row: any) {
    Metadata.getInstance<TextInput>('formCandidateData-NRCANDIDATO').value = row.NRCANDIDATO;
    Metadata.getInstance<TextInput>('formCandidateData-INCLUSAO').value = row.INCLUSAO;
    Metadata.getInstance<Select>('formCandidateData-IDSTATUSCAN').value = row.IDSTATUSCAN;
    Metadata.getInstance<Select>('formCandidateData-IDATIVO').value = row.IDATIVO;
    Metadata.getInstance<TextInput>('formCandidateData-NMCANDIDATO').value = row.NMCANDIDATO;
    Metadata.getInstance<TextInput>('formCandidateData-NMFANTCAN').value = row.NMFANTCAN;
    Metadata.getInstance<TextInput>('formCandidateData-NRINSJURCAN').value = row.NRINSJURCAN;
    Metadata.getInstance<Select>('formCandidateData-IDTPIJURCAN').value = row.IDTPIJURCAN;
    Metadata.getInstance<TextInput>('formCandidateData-DSEMAILCAN').value = row.DSEMAILCAN;
    Metadata.getInstance<TextInput>('formCandidateData-DSTELEFONECAN').value = row.DSTELEFONECAN;
    Metadata.getInstance<Select>('formSegmentData-PERFATUSEGMER').value = row.PERFATUSEGMER;
    Metadata.getInstance<Select>('formSegmentData-CDSEGMER').value = row.CDSEGMER;
    Metadata.getInstance<TextInput>('formSegmentData-NRFUNCCAN').value = row.NRFUNCCAN;
    Metadata.getInstance<TextInput>('formSegmentData-PORTCANDIDATO').value = row.PORTCANDIDATO;
    Metadata.getInstance<TextInput>('formSegmentData-QNTDUSUARIOS').value = row.QNTDUSUARIOS;
    Metadata.getInstance<TextInput>('formSegmentData-NRSISATU').value = row.NRSISATU;
    Metadata.getInstance<TextInput>('formSegmentData-QNTPONTOSACESSO').value = row.QNTPONTOSACESSO;
    Metadata.getInstance<TextInput>('formSegmentData-FISCAL').value = row.FISCAL;
    Metadata.getInstance<TextInput>('formSegmentData-CDMODATEND').value = row.CDMODATEND;
    Metadata.getInstance<TextInput>('formSegmentData-QNTDUNIDADES').value = row.QNTDUNIDADES;
    Metadata.getInstance<TextInput>('formSegmentData-NRREFEICOESDIA').value = row.NRREFEICOESDIA;
    Metadata.getInstance<TextInput>('formAdressData-NRCEPENDERECO').value = row.NRCEPENDERECO;
    Metadata.getInstance<TextInput>('formAdressData-DSENDERECO').value = row.DSENDERECO;
    Metadata.getInstance<TextInput>('formAdressData-NRIMOVELENDERECO').value = row.NRIMOVELENDERECO;
    Metadata.getInstance<TextInput>('formAdressData-DSCOMPLEENDERECO').value = row.DSCOMPLEENDERECO;
    Metadata.getInstance<TextInput>('formAdressData-DESCTITULOCAN').value = row.DESCTITULOCAN;
    Metadata.getInstance<TextInput>('formAdressData-CDPAIS').value = row.CDPAIS;
    Metadata.getInstance<TextInput>('formAdressData-SGESTADO').value = row.SGESTADO;
    Metadata.getInstance<TextInput>('formAdressData-CDMUNICIPIO').value = row.CDMUNICIPIO;
    Metadata.getInstance<TextInput>('formAdressData-CDBAIRRO').value = row.CDBAIRRO;
    Metadata.getInstance<TextInput>('formInterlocutorData-NMINTERLOCUTOR').value = row.NMINTERLOCUTOR;
    Metadata.getInstance<TextInput>('formInterlocutorData-EMAILINTERLOC').value = row.EMAILINTERLOC;
    Metadata.getInstance<TextInput>('formInterlocutorData-NRTELEFONE1').value = row.NRTELEFONE1;
    Metadata.getInstance<Select>('formDadosAdicionais-CDOPERADOR').value = row.CDOPERADOR;
    Metadata.getInstance<Select>('formDadosAdicionais-CDMODODIVULG').value = row.CDMODODIVULG;
    Metadata.getInstance<Select>('formDadosAdicionais-NMMODORECEBINFO').value = row.NMMODORECEBINFO;
    Metadata.getInstance<TextInput>('formDadosAdicionais-DSOBSERVACAOLEAD').value = row.DSOBSERVACAOLEAD;
  }

  public setFIlterEvolution() {
    const grid = Metadata.getInstance<TekGrid>('tekgrid-tab-evolution');
    grid.datasource.setFilter({
      NRCONTATO: this.currentRow.NRCONTATO,
      NRSEQCONTATO: this.currentRow.NRSEQCONTATO,
    });
    grid.datasource.get();
  }

  public checkActiveTab() {
    // Get instance of the main TekGrid
    const mainTekGrid = Metadata.getInstance<TekGrid>('tek_grid_crm_leads');

    // Get instance of the Tabs component containing tab information
    const tabs = Metadata.getInstance<Tabs>('tabsInformationLeads');

    // Get the active tab object
    const activeTab = tabs.tabs[tabs.activeTab];

    // Switch statement to handle different active tabs
    switch (activeTab.name) {
      // If the active tab is the evolution tab
      case 'tab_evolution':
        // Load evolution tab items based on the current row of the main TekGrid
        this.loadEvolutionTabItems({ row: mainTekGrid.datasource.currentRow });
        break;
      // If the active tab is the task schedule tab
      case 'tab_taskSchedule':
        // Load task schedule tab groups
        this.loadTaskScheduleTabGroups();
        break;
      // If the active tab is the interlocutor tab
      case 'tab_interlocutor':
        // Load interlocutor tab tasks
        this.loadInterlocutorTabTasks();
        break;
      // Default case (if the active tab is not recognized)
      default:
        // Do nothing
        break;
    }
  }

  private loadEvolutionTabItems({ row }: any): void {
    // Load items from the evolution tab after a delay
    setTimeout(() => {
      // Get the instance of the evolution grid
      const evolutionGrid = Metadata.getInstance<TekGrid>('tekgrid-tab-evolution');

      // Set the filter based on the selected row data
      evolutionGrid.datasource.setFilter({
        NRCONTATO: row.NRCONTATO,
        NRSEQCONTATO: row.NRSEQCONTATO,
      });

      // Load data from the evolution grid's data source
      evolutionGrid.datasource.get();
    }, 700); // Wait time before loading the data (in milliseconds)
  }

  private loadTaskScheduleTabGroups() { }

  private loadInterlocutorTabTasks() { }

  /**
   * Retrieves the instance of the TekGrid associated with CRM leads.
   * @returns The instance of the TekGrid associated with CRM leads.
   */
  private getGridInstance() {
    return Metadata.getInstance<TekGrid>('tek_grid_crm_leads');
  }

  /**
   * Retrieves the filter inputs for a specified field from the grid instance.
   *
   * This method retrieves the filter inputs associated with the specified field from the grid instance.
   * It returns the filter inputs for the specified field.
   *
   * @param {string} field - The name of the field for which to retrieve filter inputs.
   * @returns {any} The filter inputs associated with the specified field.
   */
  private getFilterInputs(field: string): any {
    return this.getGridInstance().getFilterInputs(field);
  }

  /**
   * Opens the filter modal associated with the specified component.
   *
   * This method triggers a click event on the filter button associated with the specified component,
   * opening the filter modal for that component.
   *
   * @param {any} component - The component for which to open the filter modal.
   */
  public openFilter({ component }: any): void {
    const filterButton = Metadata.getInstance<Button>(
      `${component.name}_filterButton`,
    );
    filterButton.click({} as Event, {} as HTMLElement);
  }

  /**
   * Adds a new lead by displaying a modal form.
   *
   * This method creates a modal form for adding a new lead and displays it to the user.
   *
   * @returns {Promise<void>} A promise that resolves once the modal is displayed.
   */
  public async addNewLead(): Promise<void> {
    try {
      // Create a modal form element for adding a new lead.
      this.ModalElementoAdd = await ModalService.createFromJson('/crm_leads/Modal/crm_leads_cadastro', true);

      // Display the modal form to the user.
      this.ModalElementoAdd.show();
    } catch (error) {
      // Log any errors encountered while adding a new lead.
      console.error('Error adding a new lead:', error);
    }
  }

  /**
 * Define o filtro NRPARCRESP com o valor do operador logado.
 * Carrega o indicador de carregamento, obtém o operador logado e define o valor do filtro NRPARCRESP
 * como o código do operador logado. Em seguida, oculta o indicador de carregamento.
 * Em caso de erro, registra o erro no console.
 */
  public async setFilterNRPARCRESP(): Promise<void> {
    try {
      // Exibe o indicador de carregamento
      LoadingService.show();

      // Obtém a instância do componente de seleção do operador logado
      const loggedOperInstance = Metadata.getInstance<Select>('modal-cadastro-leads-NRPARCPROPRI');

      // Obtém o código do operador logado
      const cdOperador = await this.getLoggedOperator();

      // Define o valor do filtro NRPARCRESP como o código do operador logado
      loggedOperInstance.setValue(cdOperador);

      // Oculta o indicador de carregamento
      LoadingService.hide();
    } catch (error) {
      // Registra o erro no console em caso de falha
      console.error('Error setting filter:', error);
    }
  }

  /**
   * Realiza uma busca para verificar se já existe um lead com a razão social fornecida.
   * Se existir, exibe uma mensagem de erro e limpa o valor do componente.
   * @param component O componente de entrada de texto contendo a razão social a ser verificada.
   */
  public searchParcForName({ component }: any): void {
    // Verifica se o valor do componente não é nulo
    if (component.value != null) {
      // Obtém o valor da razão social do componente de entrada de texto
      const NMCANDIDATO = Metadata.getInstance<TextInput>('modal-cadastro-leads-NMCANDIDATO').value;

      // Exibe o indicador de carregamento
      LoadingService.show();

      // Realiza uma requisição assíncrona para validar a razão social do lead
      Http.post('/leadValidate', {
        NMCANDIDATO: NMCANDIDATO.toString().replace(/ /g, '%').toUpperCase(),
      }).then((response: any) => {
        // Verifica se existem dados na resposta da requisição
        if ((response.data.data.length !== 0)) {
          // Exibe uma mensagem de erro informando que já existe um lead com esta razão social
          Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Já existe um lead com esta Razão Social!');

          // Limpa o valor do componente
          component.value = null;
        }
      }).catch((error: any) => {
        // Em caso de erro, exibe uma mensagem de erro
        Utilities.showErrorMessage(error);
      }).finally(() => {
        // Oculta o indicador de carregamento após a conclusão da requisição
        LoadingService.hide();
      });
    }
  }

  /**
   * Determines the mask format based on the value selected in the 'NRINSJURCAN' field of the modal.
   *
   * This method retrieves the formatted value of the 'NRINSJURCAN' field from the modal and
   * determines the appropriate mask format based on the selected value.
   *
   * @returns {string} - The mask format determined based on the selected value in the 'NRINSJURCAN' field.
   */
  public getMaskModal(value: string): string {
    switch (value.length) {
      case 14:
        return '##.###.###/####-##';
      case 11:
        return '###.###.###-##';
      case 6:
        return '######'; // Alterei para 6 porque o CPF tem 11 dígitos, não 6.
      case 20:
        return 'NNNNNNNNNNNNNNNNNNNN'; // Corrigi para 20 dígitos, assumindo CNPJ como exemplo.
      default:
        return '';
    }
  }

  /**
   * Validates and formats the input value for CNPJ or CPF.
   *
   * This method validates the input value for CNPJ or CPF format and updates the corresponding fields accordingly.
   *
   * @param {IEventParam<TextInput>} component - The TextInput component triggering the event.
   */
  public checkFormatCNPJ({ component }: IEventParam<TextInput>): void {
    if (component.value != null && component.value.length > 0) {
      const formattedValue = component.value.replace(/[^0-9]/g, '');
      if (formattedValue.length === 14) {
        if (!Utilities.validateCNPJ(formattedValue)) {
          Utilities.showDialogMessage('error', 'Operação bloqueada!', 'CNPJ inválido!');
          component.value = null;
        } else {
          Metadata.getInstance<Select>('modal-cadastro-leads-IDTPIJURCAN').value = 'J';
          Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN').value = formattedValue;
          this.validateRegistration(Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN'),
            Metadata.getInstance<Select>('modal-cadastro-leads-IDTPIJURCAN'));
        }
      } else if (formattedValue.length === 11) {
        // Validar CPF
        if (!Utilities.validateCPF(formattedValue)) {
          Utilities.showDialogMessage('error', 'Operação bloqueada!', 'CPF inválido!');
          component.value = null;
        } else {
          Metadata.getInstance<Select>('modal-cadastro-leads-IDTPIJURCAN').value = 'F';
          Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN').mask = '###.###.###-##';
          Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN').value = formattedValue;
          this.validateRegistration(Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN'),
            Metadata.getInstance<Select>('modal-cadastro-leads-IDTPIJURCAN'));
        }
      }
      // else {
      //   // Validar padrões internacionais
      //   if (!Utilities.validateInternationalPattern(component.value)) {
      //     Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Formato inválido!');
      //     component.value = null;
      //   }
      // }
    } else {
      Metadata.getInstance<Select>('modal-cadastro-leads-NRINSJURCAN').mask = '';
    }
  }

  public clickButtonToExpand({ component }: any) {
    const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
    form.isVisible = !form.isVisible;
    component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
  }

  public validate({ component }: any) {
    const nameForm = component.name.split('-')[0];
    this.validateCEP(`${nameForm}-NRCEPENDERECO`);
  }

  /**
   * Validates the registration of a partner based on the type of inscription and the inscription field value.
   *
   * This method validates the registration of a partner based on the
   * type of inscription (e.g., CNPJ or CPF)
   * and the corresponding inscription field value.
   * It sends a request to the server to check if there is already
   * a partner associated with the provided inscription. If a partner with the same inscription exists, it displays
   * an error message.
   *
   * @param {any} inscriptionField - The field containing the inscription value (e.g., NRINSJURCAN).
   * @param {any} typeIncription - The field containing the type of inscription (e.g., IDTPIJURCAN).
   */
  public validateRegistration(inscriptionField: any, typeIncription: any): void {
    if (inscriptionField.value != null && typeIncription.value != null) {
      const IDTPIJURCAN = typeIncription.value;
      const NRINSJURCAN = inscriptionField.value;
      LoadingService.show();
      Http.post('/leadValidate', {
        IDTPIJURCAN,
        NRINSJURCAN,
      }).then((response: any) => {
        if ((response.data.data.length !== 0)) {
          Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Já existe um Parceiro atrelado a este CNPJ');
          inscriptionField.value = null;
        }
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }

  /**
   * Handles the event triggered when a filter grid is created.
   *
   * This method is called when a filter grid component is created,
   * allowing dynamic filtering of data.
   * It sets up dynamic filters based on the current user's operator
   * ID and the active status of the records.
   *
   * @param {any} component The created filter grid component object.
   * @returns {Promise<void>}
   */
  public async onCreatedFilterGrid({ component }: any): Promise<void> {
    try {
      (component.datasource as TekRestDatasource).dynamicFilter = {
        IDOPERATIVO: [{
          relation: 'AND',
          operation: 'EQUALS',
          value: 'A',
        }],
        IDATIVO: [{
          relation: 'AND',
          operation: 'EQUALS',
          value: 'S',
        }],
        CDOPERADOR: [{
          relation: 'AND',
          operation: 'IN',
          value: [localStorage.getItem('CDOPERADOR')],
        }],
      };
    } catch (error) {
      console.error('Erro ao definir dynamicFilter:', error);
    }
  }

  /**
   * Handles changes in the status of an operator.
   *
   * This method is triggered when there's a change in the status of an operator component.
   * It filters the options of the responsible operator field based on the selected status
   * of the operator.
   *
   * @param {any} component The changed component object containing the name and value.
   * @returns {void}
   */
  public changeStatusOper({ component }: any): void {
    const statusOper = this.getFilterInputs('IDOPERATIVO');
    const respOperator = this.getFilterInputs('CDOPERADOR');

    const respOperatorInstance = Metadata.getInstance<SelectMultiple>(respOperator[0].name);

    if (component.value) {
      respOperatorInstance.datasource.setFilter({
        IDOPERATIVO: statusOper[0].value,
      });
    } else {
      respOperatorInstance.datasource.setFilter({
        IDOPERATIVO: 'A',
      });
    }
  }

  /**
   * Handles changes in the state field based on the selected country.
   *
   * This method is triggered when there's a change in the value of a component.
   * If the changed component is the country field, it enables the state
   * field and filters its options based on the selected country.
   * If the changed component is the country field within a modal, it
   * behaves similarly to the previous case, but with modal-specific field instances.
   * If the changed component is neither the country field nor within a modal,
   * it disables the state field.
   *
   * @param {any} component The changed component object containing the name and value.
   * @returns {void}
   */

  public changeFieldState({ component }: any): void {
    const statusFieldCountry = this.getFilterInputs('CDPAIS');
    const statusFieldState = this.getFilterInputs('SGESTADO');

    const stateFieldInstance = Metadata.getInstance<SelectMultiple>(statusFieldState[0].name);

    if (component.value != null && component.name === statusFieldCountry[0].name) {
      stateFieldInstance.disabled = false;
      stateFieldInstance.datasource.setFilter({
        CDPAIS: statusFieldCountry[0].value,
      });
    } else if (component.value != null && component.name === 'modal-cadastro-leads-CDPAIS') {
      const modalFieldCountry = Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS');
      const modalFieldState = Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO');
      modalFieldState.disabled = false;
      modalFieldState.datasource.setFilter({
        CDPAIS: modalFieldCountry.value,
      });
    } else {
      stateFieldInstance.disabled = true;
      stateFieldInstance.value = null;
    }
  }

  /**
   * Handles changes in the municipality field based on the selected state.
   *
   * This method is triggered when there's a change in the value of a component.
   * If the changed component is the state field, it enables the municipality field and filters
   * its options based on the selected country and state.
   * If the changed component is the state field within a modal, it behaves similarly to the
   * previous case, but with modal-specific field instances.
   * If the changed component is neither the state field nor within a modal,
   * it disables the municipality field.
   *
   * @param {any} component The changed component object containing the name and value.
   * @returns {void}
   */

  public changeInTheMunicipalityField({ component }: any) {
    const statusFieldCountry = this.getFilterInputs('CDPAIS');
    const statusFieldState = this.getFilterInputs('SGESTADO');
    const statusFieldMunicipality = this.getFilterInputs('CDMUNICIPIO');

    const municipalityFieldInstance = Metadata.getInstance<SelectMultiple>(statusFieldMunicipality[0].name);

    if (component.value != null && component.name === statusFieldState[0].name) {
      municipalityFieldInstance.disabled = false;
      municipalityFieldInstance.datasource.setFilter({
        CDPAIS: statusFieldCountry[0].value,
        SGESTADO: statusFieldState[0].value,
      });
    } else if (component.value != null && component.name === 'modal-cadastro-leads-SGESTADO') {
      const modalFieldCountry = Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS');
      const modalFieldState = Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO');
      const modalFieldMunicipality = Metadata.getInstance<Select>('modal-cadastro-leads-CDMUNICIPIO');
      modalFieldMunicipality.disabled = false;
      modalFieldMunicipality.datasource.setFilter({
        CDPAIS: modalFieldCountry.value,
        SGESTADO: modalFieldState.value,
      });
    } else {
      municipalityFieldInstance.disabled = true;
      municipalityFieldInstance.value = null;
    }
  }

  /**
   * Handles changes in the neighborhood field based on the selected municipality.
   *
   * This method is triggered when there's a change in the value of a component.
   * If the changed component is the municipality field, it enables the neighborhood
   * field and filters its options based on the selected country, state, and municipality.
   * If the changed component is the municipality field within a modal, it behaves
   * similarly to the previous case, but with modal-specific field instances.
   * If the changed component is neither the municipality field nor within a modal,
   * it disables the neighborhood field.
   *
   * @param {any} component The changed component object containing the name and value.
   * @returns {void}
   */
  public changeInTheNeighborhoodField({ component }: any): void {
    const statusFieldCountry = this.getFilterInputs('CDPAIS');
    const statusFieldState = this.getFilterInputs('SGESTADO');
    const statusFieldNeighborhood = this.getFilterInputs('CDBAIRRO');
    const statusFieldMunicipality = this.getFilterInputs('CDMUNICIPIO');

    const neighborhoodFieldInstance = Metadata.getInstance<SelectMultiple>(statusFieldNeighborhood[0].name);

    if (component.value != null && component.name === statusFieldMunicipality[0].name) {
      neighborhoodFieldInstance.disabled = false;
      neighborhoodFieldInstance.datasource.setFilter({
        CDPAIS: statusFieldCountry[0].value,
        SGESTADO: statusFieldState[0].value,
        CDMUNICIPIO: statusFieldMunicipality[0].value,
      });
    } else if (component.value != null && component.name === 'modal-cadastro-leads-CDMUNICIPIO') {
      const modalFieldCountry = Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS');
      const modalFieldState = Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO');
      const modalFieldNeighborhood = Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO');
      const modalFieldMunicipality = Metadata.getInstance<Select>('modal-cadastro-leads-CDMUNICIPIO');
      modalFieldNeighborhood.disabled = false;
      modalFieldNeighborhood.datasource.setFilter({
        CDPAIS: modalFieldCountry.value,
        SGESTADO: modalFieldState.value,
        CDMUNICIPIO: modalFieldMunicipality.value,
      });
    } else {
      neighborhoodFieldInstance.disabled = true;
      neighborhoodFieldInstance.value = null;
    }
  }

  /**
   * Displays a dialog box with buttons and a grid, to confirm whether the user
   * wants to find the address based on the information provided in the field.
   * @param {Object} component - The component used.
   * @param {Object} options - The options for the dialog.
   * @param {any} options.component - The component to be displayed in the dialog.
   */
  public showDialogWithButtonsGrid({ component }: any) {
    if (component.value.length === 8) {
      DialogService.show({
        name: 'dialog-find-partner-addres',
        type: 'warning',
        title: 'Buscar Endereço',
        text: 'Deseja buscar o endereço pelo CEP?',
        buttons: [
          {
            name: 'NAO',
            component: 'ZdButton',
            label: 'Não',
            outline: true,
            events: {
              click: () => {
                DialogService.hide();
              },
            },
          },
          {
            name: 'SIM',
            component: 'ZdButton',
            label: 'Sim',
            events: {
              click: () => {
                this.validateCEP(component.name);
                DialogService.hide();
              },
            },
          },
        ],
      });
    }
  }

  /**
   * Validates the CEP (postal code).
   * If the CEP is valid, it searches for the address details and updates the
   * corresponding form fields.
   */
  public async validateCEP(nameForm: string) {
    const CEP = Metadata.getInstance<TextInput>(`${nameForm}`).value.replace('-', '');
    if (CEP.length !== 8) {
      return;
    }

    LoadingService.show();
    try {
      const response = await this.searchCep(CEP) as IDictionary;
      const addressFound = response.data.dataset.ENDERECO;

      if (!addressFound) {
        return;
      }

      // Limpar campos de endereço antes de preencher com novos dados
      this.clearAddressFields();

      Metadata.getInstance<TextInput>('modal-cadastro-leads-DSENDERECO').value = addressFound.DSENDERECO;
      Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS').setValue(addressFound.CDPAIS ?? '');
      await Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO').setValue(addressFound.SGESTADO ?? '');
      await Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO').setValue(addressFound.CDBAIRRO ?? '');
      await Metadata.getInstance<Select>('modal-cadastro-leads-CDMUNICIPIO').setValue(addressFound.CDMUNICIPIO ?? '');

      // Se o código do bairro estiver vazio, busca e preenche
      if (!addressFound.CDBAIRRO) {
        await this.fetchAndSetBairroCode(addressFound);
      }
    } catch (error) {
      Utilities.showErrorMessage(error);
    } finally {
      LoadingService.hide();
    }
  }

  /**
   * Clears the address fields in the modal form.
   */
  private clearAddressFields() {
    Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS').setValue('');
    Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO').setValue('');
    Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO').setValue('');
    Metadata.getInstance<Select>('modal-cadastro-leads-CDMUNICIPIO').setValue('');
  }

  /**
   * Fetches the bairro code based on the provided address information and sets
   * the value in the metadata.
   * @param addressFound - The address information containing NMBAIRRO, CDPAIS, SGESTADO, and CDMUNICIPIO.
   */
  private async fetchAndSetBairroCode(addressFound: IDictionary) {
    try {
      const bairroResponse = await Http.post('/findCodeDistrict', {
        NMBAIRRO: addressFound.NMBAIRRO,
        CDPAIS: addressFound.CDPAIS,
        SGESTADO: addressFound.SGESTADO,
        CDMUNICIPIO: addressFound.CDMUNICIPIO,
      });
      const bairroData = bairroResponse.data.data[0];
      const CDBAIRRO = bairroData ? bairroData.CDBAIRRO : '';

      Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO').setValue(CDBAIRRO);
    } catch (error) {
      Utilities.showErrorMessage(error);
    }
  }

  // Faz uma busca pelo CEP através da API do Correio, e retorna o endereço em formato de promise.
  private searchCep(cepValue: number) {
    if (String(cepValue).length === 8) {
      return Http.post('/lib_findAddress', {
        requestType: 'Row',
        row: { CEP: cepValue },
      });
    }
    return 0;
  }

  /**
   * Saves a new lead.
   *
   * This method sends a POST request to the '/saveNewLeads' endpoint with the lead data.
   * If the form is validated successfully, it sends the lead data to the server to be saved.
   * Upon successful saving, it closes the modal window, refreshes the leads grid, and displays a success message.
   * If an error occurs during the process, it displays an error message.
   */
  public saveNewLead(): void {
    const form = Metadata.getInstance<Form>('modal-cadastro-leads-form');
    const grid = Metadata.getInstance<TekTreeGrid>('tek_grid_crm_leads');

    if (Utilities.validateForm(form)) {
      LoadingService.show();
      Http.post('/saveNewLeads', {
        NMCANDIDATO: Metadata.getInstance<TextInput>('modal-cadastro-leads-NMCANDIDATO').value,
        NMFANTCAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-NMFANTCAN').value,
        NRINSJURCAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRINSJURCAN').value,
        IDTPIJURCAN: Metadata.getInstance<Select>('modal-cadastro-leads-IDTPIJURCAN').value,
        IDATIVO: Metadata.getInstance<Select>('modal-cadastro-leads-IDATIVO').value,
        IDSTATUSCAN: Metadata.getInstance<Select>('modal-cadastro-leads-IDSTATUSCAN').value,
        DSEMAILCAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-DSEMAILCAN').value,
        DSTELEFONECAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-DSTELEFONECAN').value,
        PERFATUSEGMER: Metadata.getInstance<TextInput>('modal-cadastro-leads-PERFATUSEGMER').value,
        CDSEGMER: Metadata.getInstance<Select>('modal-cadastro-leads-CDSEGMER').value,
        NRFUNCCAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRFUNCCAN').value,
        NRSISATU: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRSISATU').value,
        NRCEPENDERECO: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRCEPENDERECO').value,
        DSENDERECO: Metadata.getInstance<TextInput>('modal-cadastro-leads-DSENDERECO').value,
        NRIMOVELENDERECO: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRIMOVELENDERECO').value,
        DSCOMPLEENDERECO: Metadata.getInstance<TextInput>('modal-cadastro-leads-DSCOMPLEENDERECO').value,
        CDPAIS: Metadata.getInstance<Select>('modal-cadastro-leads-CDPAIS').value,
        SGESTADO: Metadata.getInstance<Select>('modal-cadastro-leads-SGESTADO').value,
        CDMUNICIPIO: Metadata.getInstance<Select>('modal-cadastro-leads-CDMUNICIPIO').value,
        NMBAIRROCAN: Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO').displayValue,
        CDBAIRRO: Metadata.getInstance<Select>('modal-cadastro-leads-CDBAIRRO').value,
        NMINTERLOCUTOR: Metadata.getInstance<Select>('modal-cadastro-leads-NMINTERLOCUTOR').value,
        EMAILINTERLOC: Metadata.getInstance<TextInput>('modal-cadastro-leads-EMAILINTERLOC').value,
        NRTELEFONE1: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRTELEFONE1').value,
        CDOPERADOR: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRPARCPROPRI').value,
        NRPARCPROPRI: Metadata.getInstance<Select>('modal-cadastro-leads-NRPARCPROPRI').datasource.data[0].NRPARCRESP,
        NRSEQPERFATIV: Metadata.getInstance<Select>('modal-cadastro-leads-text-NRSEQPERFATIV').value,
        CDMODODIVULG: Metadata.getInstance<Select>('modal-cadastro-leads-text-CDMODODIVULG').value,
        NRMODORECEBINFO: Metadata.getInstance<Select>('modal-cadastro-leads-text-NRMODORECEBINFO').value,
        DESCTITULOCAN: Metadata.getInstance<TextInput>('modal-cadastro-leads-DESCTITULOCAN').value,
        DSOBSERVACAOLEAD: Metadata.getInstance<TextInput>('modal-cadastro-leads-DSOBSERVACAOLEAD').value,
        PORTCANDIDATO: Metadata.getInstance<TextInput>('modal-cadastro-leads-PORTCANDIDATO').value,
        QNTDUSUARIOS: Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTDUSUARIOS').value,
        QNTDUNIDADES: Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTDUNIDADES').value,
        QNTPONTOSACESSO: Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTPONTOSACESSO').value,
        FISCAL: Metadata.getInstance<TextInput>('modal-cadastro-leads-FISCAL').value,
        CDMODATEND: Metadata.getInstance<TextInput>('modal-cadastro-leads-CDMODATEND').value,
        NRREFEICOESDIA: Metadata.getInstance<TextInput>('modal-cadastro-leads-NRREFEICOESDIA').value,
      }).then(() => {
        this.closeModalCadastroLeads();
        grid.datasource.get();
        Utilities.showSuccessMessage();
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }

  /**
   * changingVisibilityBasedOnProfile
   */
  public changingVisibilityBasedOnProfile({ component }: any) {
    const activityProfile = component.value;

    if (activityProfile === '2') {
      this.showProfile2Fields();
    } else if (activityProfile === '4') {
      this.showProfile4Fields();
    } else {
      this.hideAllFields();
    }
  }

  private showProfile2Fields() {
    Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTDUNIDADES').isVisible = true;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTPONTOSACESSO').isVisible = true;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-FISCAL').isVisible = true;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-CDMODATEND').isVisible = true;
  }

  private showProfile4Fields() {
    Metadata.getInstance<TextInput>('modal-cadastro-leads-NRREFEICOESDIA').isVisible = true;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTDUNIDADES').isVisible = true;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTPONTOSACESSO').isVisible = false;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-FISCAL').isVisible = false;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-CDMODATEND').isVisible = false;
  }

  private hideAllFields() {
    Metadata.getInstance<TextInput>('modal-cadastro-leads-NRREFEICOESDIA').isVisible = false;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTDUNIDADES').isVisible = false;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-QNTPONTOSACESSO').isVisible = false;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-FISCAL').isVisible = false;
    Metadata.getInstance<TextInput>('modal-cadastro-leads-CDMODATEND').isVisible = false;
  }

  public modifiVisibilityBasedOnStatusLead(row: any) {
    const statusLead = row;

    // Set the visibility of the buttons based on the lead status
    if (statusLead.IDSTATUSCAN === '3') {
      this.setButtonVisibility(false, true);
    } else if (statusLead.IDSTATUSCAN === '4') {
      this.setButtonVisibility(true, false);
    } else {
      this.setButtonVisibility(true, true);
    }
  }

  // Helper method to set the visibility of the buttons
  private setButtonVisibility(qualifiedVisibility: boolean, desqualifiedVisibility: boolean) {
    this.buttonQualifiedyVisibility = qualifiedVisibility;
    this.buttonDesqualifiedyVisibility = desqualifiedVisibility;
  }

  /**
   * Changes the status of a lead based on the provided component.
   * If the component represents the action to qualify a lead,
   * the lead status will be updated to 'Qualified' (IDSTATUSCAN: '3').
   * If the component represents the action to disqualify a lead,
   * the lead status will be updated to 'Disqualified' (IDSTATUSCAN: '4').
   * @param {any} { component } The component triggering the status change action.
   */
  public changeLeadStatus({ component }: any) {
    const isQualifyAction = (component.name === 'actionQualifyLead');
    const newLeadStatus = isQualifyAction ? '3' : '4';

    this.updateLeadStatus(newLeadStatus);
  }

  private updateLeadStatus(newStatus: string) {
    LoadingService.show();

    const requestData = {
      NRPARCNEGOCIO: this.currentRow.NRPARCNEGOCIO,
      IDSTATUSCAN: newStatus,
      NRORG: this.currentRow.NRORG,
      CRM_NRORG: this.currentRow.NRORG,
      CDOPERULTATU: this.currentRow.CDOPERADOR,
    };

    Http.post('/changeStatusLead', requestData)
      .then(() => {
        Utilities.showSuccessMessage();
        this.getGridInstance().datasource.get();
        Metadata.getInstance<Select>('formCandidateData-IDSTATUSCAN').setValue(newStatus);
      })
      .catch((error: any) => {
        Utilities.showErrorMessage(error);
      })
      .finally(() => {
        LoadingService.hide();
      });
  }
}
