/* eslint-disable */
import {
    Http,
    Metadata,
} from '@zeedhi/core';
import {
    DialogService, AlertService, FileInput, Form, LoadingService,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Utilities {
    public static readonly MSG_CAMPOS_NAO_PREENCHIDOS = 'Existem campos obrigatórios não preenchidos!';

    public static alertMessage(message: string, type: string) {
        AlertService.show({
            name: 'basic-alert',
            text: message,
            top: true,
            left: false,
            timeout: 3000,
            color: type,
        });
    }

    public static showErrorMessage(error: any) {
        console.error(error);
        let msg = '';
        if (error.response && error.response.data && error.response.data.error) {
            msg = error.response.data.error;
        } else {
            msg = error;
        }
        if (!msg) {
            msg = 'Ocorreu um erro inesperado. Por favor, entre em contato com o suporte.';
        }

        if (msg.includes("ORA-02292")) {
            msg = "Exclusão não permitida! O registro possui registros filhos.";
        }

        this.showDialogMessage('error', 'Erro', msg);
    }

    /**
     * Mostra uma caixa de mensagem dialog.
     * 
     * @param type tipo da mensagem (success, info, warning, error)
     * @param title titulo da caixa de mensagem
     * @param text conteudo da caixa de mensagem
     */
    public static showDialogMessage(type: string, title: string, text: string) {
        DialogService.show({
            name: type + '_dialog',
            type: type,
            title: title,
            text: text,
        });
    }

    public static async salvarDados(rota: any, parametros: any) {
        await Http.post(rota, parametros).then(() => {
            this.showSuccessMessage();
        }).catch((error: any) => {
            this.showErrorMessage(error);
        });
    }

    public static showSuccessMessage() {
        AlertService.show({
            name: 'basic-alert',
            text: 'Processo realizado com sucesso!',
            top: true,
            left: false,
            color: 'success',
            timeout: 2000,
            buttonsSlot: [
                {
                    name: 'undo-button',
                    component: 'ZdButton',
                    icon: true,
                    right: true,
                    iconName: 'close',
                    color: 'white',
                    events: {
                        click: () => { AlertService.hide(); },
                    },
                },
            ],
        });
    }

    public static async buildArrayToUpload(nomeComponente: string) {
        const file = Metadata.getInstance<FileInput>(nomeComponente).value;
        if (file) {
            const base64file = await this.toBase64(file);
            return [{
                b64File: base64file,
                lastModified: file.lastModified,
                name: file.name,
                size: file.size,
                type: file.type,
                webkitRelativePath: file.webkitRelativePath,
            }];
        }
        return [];
    }

    /* public static async buildArrayToUploadPromise(nomeComponente: string) {
        const file = Metadata.getInstance<FileInput>(nomeComponente).value;
        return new Promise((resolve, reject) => {
            if (file) {
                this.toBase64(file).then((base64file) => {
                    resolve([{
                        b64File: base64file,
                        lastModified: file.lastModified,
                        name: file.name,
                        size: file.size,
                        type: file.type,
                        webkitRelativePath: file.webkitRelativePath,
                    }]);
                }).catch((error) => {
                    reject(error);
                });
            }
            else {
                resolve([]);
            }
        });
    }; */

    public static async toBase64(file: any) {
        return new Promise((resolve, reject) => {
            const reader = new FileReader();
            reader.readAsDataURL(file);
            reader.onload = () => resolve(reader.result);
            reader.onerror = (error) => reject(error);
        });
    }

    public static showConfirmDeleteMessage(event: any, functionConfirm: string) {
        event.preventDefault();
        functionConfirm = '{{' + functionConfirm + '}}';
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'button-no',
                    component: 'ZdButton',
                    label: 'Não',
                    events: {
                        click: () => DialogService.hide(),
                    },
                    outline: true,
                },
                {
                    name: 'button-yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: functionConfirm
                    },
                },
            ],
        });
    }

    public static getLoggedOperator() {
        return Http.get('/getLoggedOperator', {})
            .then((response) => response)
            .catch((error) => {
                console.error('Erro ao obter operador logado:', error);
                throw error; // Rejeitar a promessa para que os erros possam ser tratados externamente
            });
    }

    public static validateForm(form: Form) {
        if (form.validate()) {
            return true;
        }
        this.showAlertCamposObrigatorios();
        return false;
    }

    public static validateGrid(grid: TekGrid) {
        if (grid.isGridValid()) {
            return true;
        }
        this.showAlertCamposObrigatorios();
        return false;
    }

    public static showAlertCamposObrigatorios() {
        this.alertMessage(this.MSG_CAMPOS_NAO_PREENCHIDOS, 'error');
    }

    public static async timeout(time: number) {
        return new Promise((resolve) => setTimeout(resolve, time));
    }

    public static async doAsync(callBack: Function, time: number) {
        return new Promise<void>((resolve) => {
            this.timeout(time).then(() => {
                callBack();
                resolve();
            });
        });
    }

    public static validateCNPJ(cnpj: string): boolean {
        if (!cnpj || cnpj.length !== 14 || /^(.)\1+$/.test(cnpj)) {
            return false;
        }

        let sum = 0;
        let peso = 5;
        for (let i = 0; i < 12; i++) {
            sum += parseInt(cnpj.charAt(i)) * peso--;
            if (peso < 2) peso = 9;
        }

        let mod = sum % 11;
        let dv1 = mod < 2 ? 0 : 11 - mod;

        sum = 0;
        peso = 6;
        for (let i = 0; i < 13; i++) {
            sum += parseInt(cnpj.charAt(i)) * peso--;
            if (peso < 2) peso = 9;
        }

        mod = sum % 11;
        let dv2 = mod < 2 ? 0 : 11 - mod;

        return parseInt(cnpj.charAt(12)) === dv1 && parseInt(cnpj.charAt(13)) === dv2;
    }

    public static validateCPF(cpf: string): boolean {
        if (!cpf || cpf.length !== 11 || /^(.)\1+$/.test(cpf)) {
            return false;
        }

        let sum = 0;
        let weight = 10;
        for (let i = 0; i < 9; i++) {
            sum += parseInt(cpf.charAt(i)) * weight--;
        }

        let mod = sum % 11;
        let digit1 = mod < 2 ? 0 : 11 - mod;

        sum = 0;
        weight = 11;
        for (let i = 0; i < 10; i++) {
            sum += parseInt(cpf.charAt(i)) * weight--;
        }

        mod = sum % 11;
        let digit2 = mod < 2 ? 0 : 11 - mod;

        return parseInt(cpf.charAt(9)) === digit1 && parseInt(cpf.charAt(10)) === digit2;
    }

    public static validateInternationalPattern(value: string, pattern: RegExp): boolean {
        return pattern.test(value);
    }
}
